<?php
/**
 * clase_zen_html_modelo_datos.php
 * @author Juan Belon
 * @access public
 * @copyright LGPL, GPL
 * @package zenphp
 * @version 0.1.1
 * @uses zenphp FrameWork
 * @link http://csl2-zenphp.forja.rediris.es
 * @link http://www.zenphp.es
 * @link https://forja.rediris.es/projects/csl2-zenphp/
 * @magic Clase que genera un visualizador HTML para un modelo de datos 
 * @see zen_andamio La clase scaffolding de zenphp que expande esta
 */
// +----------------------------------------------------------------------
// | zenphp.es
// +----------------------------------------------------------------------
if (!defined('ZF_SEGURO_ZEN')) die(_('Acceso directo no permitido'));
class zen_html_modelo_datos {
    /**
     * Clase modelo de datos asociada a la clase HTML
     *
     * @var zen_modelo_datos
     */
    var $padre;
    /**
     * Constructor
     *
     * @param zen_modelo_datos $_padre
     * @return zen_html_modelo_datos
     */
    function zen_html_modelo_datos(&$_padre){
        $this->padre =& $_padre;
        //Esta clase depende sobre todo de las plantillas
        if (!zen___carga_clase('zen_plantilla')) die(_('No se pudo cargar la clase plantillas de zenphp'));
    }
    /**
     * Utilizando una clase plantilla y un fichero HTML devuelve el contenido relleno con los datos.
     * Si rellenamos $fichero_base ,se colocan los datos del listado dentro de la base ,sustituyendolo por la $marca_base
     * Si alguna de $reemplazos_base o $reemplazos_datos tiene contenido, se reemplazan previamente en su correspondiente plantilla
     *
     * @param str $fichero_elementos
     * @param str $fichero_base
     * @param str $marca_base
     * @param array|null $reemplazos_base
     * @param array|null $reemplazos_datos
     * @return str : html
     */
    function listado($fichero_elementos,$fichero_base="",$marca_base="",$reemplazos_base=null,$reemplazos_datos=null){
        $p        =& new zen_plantilla($fichero_elementos);
        $datos = $this->padre->obtener();
        if (is_array($reemplazos_datos)) $p->pasarAplantilla($reemplazos_datos);
        $aux   = $p->contenido;
        $n     = sizeof($datos);
        $html  = "";
        
        for ($i=0; $i<$n; $i++){
            $p->contenido = $aux;
            $p->pasarAplantilla($datos[$i]);
            $html .= $p->contenido;
        }
        if (!empty($fichero_base) && !empty($marca_base)){
            $p2= $p;
            $p2->cargar($fichero_base);
            if (is_array($reemplazos_base))    $p2->pasarAplantilla($reemplazos_base);
            $p2->reemplazar($marca_base , $html);
            return $p2->contenido;
        } else return $html;
    }
    /**
     * Obtiene un registro de la BD con id de la lista de campos de las tablas y devuelve el HTML. Necesita la plantilla donde se reemplazan los datos...
     * El $fichero_plantilla se carga para meterle los datos del modelo_datos y los $reemplazos son un array que se reemplaza previamente en la plantilla por si le falta algo extra al modelo_datos
     *
     * @param str $fichero_plantilla
     * @param str $reemplazos 
     * @return str :html
     */
    function editar($fichero_plantilla,&$reemplazos){
        $p =& new zen_plantilla();        
        $c = split(",",$this->padre->campos);
        $tupla = array();
        $nombre = str_replace("`","",$c[0]); //Fix: nombres de campo con `
        if (!$p->cargar($fichero_plantilla)) return false;
        if (isset($_REQUEST[$nombre]))
        { //Para los arrays asociativos se eliminan los caracteres ` de campos, en SQL no
            $tupla = $this->padre->obtener($this->padre->campos,""," where ".
                     $c[0]."=".intval($_REQUEST[$nombre]),""," limit 1");
            if (!$tupla) return ZF_TEXT_SIN_DATOS;
        }
        //Primero los reemplazos de usuario
        $p->pasarAplantilla($reemplazos);
        //Despues los datos del modelo de datos:
        if (count($tupla)>0)
         $p->pasarAplantilla($tupla[0]);
        return $p->contenido;
    }
    /**
     * Muestra un registro en concreto usando una plantilla
     *
     * @param int $id
     * @param str $fichero_plantilla
     * @param array|null $reemplazos
     * @return str : html
     */
    function mostrar($id,$fichero_plantilla,$reemplazos=null){
        if (!$this->padre->existe($id)) {
            return ZF_TEXT_SIN_DATOS; //cte en zenphp/contenido/{idioma}/
        }
        
        $p =& new zen_plantilla();
        $campos = split(",",$this->padre->campos);        
        if ($reemplazos==null || !is_array($reemplazos)) $reemplazos = array();
        return $p->contenido_reemplaza($fichero_plantilla, //Mete en la plantilla el contenido
        array_merge( //de los arrays de datos con el identificador y los reemplazos previos juntos
            $this->padre->obtener_primero("","","where ".$campos[0]."=".intval($id),"","limit 1"),
            $reemplazos
        ));
    }
    /**
     * Muestra el contenido de un listado por defecto usando como plantilla el {nombre de la clase $padre}.html dentro del directorio con el mismo nombre
     * donde debe de haber una etiqueta #elementos_{nombre de la clase $padre}# que es reemplazada por los elementos procesados con la plantilla 
     * del fichero {nombre de la clase $padre}_elementos.html. Ej, si el modelo de datos se llama noticias
     * el directorio es: /media/plantillas/es/noticias/
     * dentro está noticias.html con una etiqueta en el que diga: #noticias_elementos# 
     * que es reemplazada por noticias_elementos.html que contiene los campos de la tabla noticias entre almohadillas #
     * ejemplo : <h1>#id_noticia# : #titulo_es#</h1> donde id_noticia,titulo_es son campos de la tabla noticias.
     */
    function index(){
        //Listado por defecto, para ello vamos a tomar la plantilla
        $nombre_plantilla = get_class($this->padre); //clase noticias por ejemplo
        //Incluir el nombre de la clase para el directorio
        $directorio = $nombre_plantilla.DIRECTORY_SEPARATOR.$nombre_plantilla;//ej:noticias/noticias[...]
        if (!is_readable($directorio."_elementos.html") ||!is_readable($directorio.".html")){
            trigger_error(_("No se pueden leer los ficheros de plantillas para mostrar el listado de este modelo: ").$directorio."_elementos.html , ".$directorio.".html",E_USER_NOTICE);
        } else {
         //Muestra el listado con la plantilla base /media/plantillas/es/noticias/noticias.html y la plantilla noticias_elementos,del mismo dir.
         return $this->listado($directorio."_elementos.html",$directorio.".html","elementos_".$nombre_plantilla);
        }
    }
}
?>